"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tree = void 0;
const node_1 = require("./node");
class Tree {
    constructor(leaves) {
        this.leaves = leaves.sort((a, b) => a.gte(b));
    }
    get root() {
        let nodes = this.leaves.map((leaf) => new node_1.Node(leaf.value));
        if (nodes.length === 0)
            throw new Error('Cannot generate merkle root from empty tree.');
        if (nodes.length === 1)
            return nodes[0];
        while (nodes.length > 1) {
            const temp = [];
            for (let i = 0; i < nodes.length; i += 2) {
                if (!nodes[i + 1]) {
                    temp.push(nodes[i]);
                    break;
                }
                temp.push(nodes[i].hash(nodes[i + 1]));
            }
            nodes = temp;
        }
        return nodes[0];
    }
    prove(leaf) {
        let proof = [];
        let nodes = this.leaves.map((leaf) => new node_1.Node(leaf.value));
        let nodeProved = new node_1.Node(leaf.value);
        const merkleTreeLevel = Math.ceil(Math.log2(nodes.length));
        if (nodes.length === 0)
            throw new Error('Cannot find prove from empty tree!');
        if (nodes.length === 1)
            return proof;
        let sibling = undefined;
        for (let i = 0; i < merkleTreeLevel; i++) {
            let provedIdx = nodes.findIndex((node) => node.eq(nodeProved));
            if (provedIdx % 2 === 0 && provedIdx + 1 < nodes.length) {
                sibling = nodes[provedIdx + 1];
            }
            else {
                sibling = nodes[provedIdx - 1];
            }
            if (sibling) {
                nodeProved = nodeProved.hash(sibling);
                proof.push(sibling);
            }
            const nextLevel = [];
            for (let i = 0; i < nodes.length; i += 2) {
                if (i + 1 < nodes.length)
                    nextLevel.push(nodes[i].hash(nodes[i + 1]));
                else
                    nextLevel.push(nodes[i]);
            }
            nodes = nextLevel;
        }
        return proof;
    }
    verify(leaf, proof) {
        let node = new node_1.Node(leaf.value);
        for (let i = 0; i < proof.length; i++)
            node = node.hash(proof[i]);
        return this.root.eq(node);
    }
}
exports.Tree = Tree;
//# sourceMappingURL=tree.js.map